/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ncbricks.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

static int parse_int(const char* const str,
	const int minv, /* minimal value */
	const int defv, /* default value */
	const int maxv) /* maximal value */
{
	char* end = NULL;
	const long val = strtol(str, &end, 0);

	if(NULL==end || *end)
	{
		/* the string was invalid */
		return defv;
	}

	if(minv>val) { return minv; }
	if(maxv<val) { return maxv; }
	return (int)val;
}

static bool parse_bool(const char* const str)
{
	return 0==strcmp(str, "yes") || 0==strcmp(str, "true")
		 || 0==strcmp(str, "on") || 0!=strtol(str, NULL, 0);
}

void cfgfile_load(const char* const path, const bool tolerant)
{
	FILE* const file = fopen(path, "rt");
	char linebuf[200];
	char* line;

	if(NULL==file)
	{
		/* file was not found, or couldn't be opened, but we ignore this */
		return;
	}

	while(NULL!=(line=fgets(linebuf, sizeof(linebuf), file)))
	{
		char* param;

		/* trim the line */
		size_t len;
		while(*line && isspace(*line)) { ++line; }
		len = strlen(line);
		while(0<len && isspace(line[len-1])) { line[--len] = '\0'; }

		/* skip empty and comment lines */
		if(0==len || '#'==*line) { continue; }

		/* find parameter */
		if(NULL==(param=strchr(line, ' ')))
		{
			/* ill-formed command in configuration file */
			if(!tolerant)
			{
				fprintf(stderr, "%s: ill-formed command: \"%s\"\n",
					path, line);
				exit(3);
			}
			continue;
		}
		*param++ = '\0';

		if(0==strcmp(line, "balls"))
		{
			g_start_balls = parse_int(param, 1, INITIAL_BALLS, MAX_LIFE);
		}
		else
		if(0==strcmp(line, "ammo"))
		{
			g_start_ammo = parse_int(param, 0, INITIAL_AMMO, MAX_AMMO);
		}
		else
		if(0==strcmp(line, "colors"))
		{
			g_use_colors = parse_bool(param);
		}
		else
		if(0==strcmp(line, "no_evil_random_charms"))
		{
			g_rnd_charm_only_good = parse_bool(param);
		}
		else
		{
			/* unknown token */
			if(!tolerant)
			{
				fprintf(stderr, "%s: unknown token: \"%s\"\n", path, line);
				exit(3);
			}
		}
	}

	fclose(file);
}
